-- This is a parser for OBJ files. Do not touch unless you know what you're doing.
local NAME=...

local T = love.filesystem.load('strManip.lua')
local strManip = T
local split = strManip(0).split
local starts = strManip(0).starts
local ends = strManip(0).ends

local POBJ = { }

function parseObj(objFilePath) -- Parses the OBJ file at objFilePath
    local contents, size = love.filesystem.read(objFilePath, all) -- Read the OBJ file
    local name = "" -- The name specified in the OBJ file (Not the actual file name)
    local step1 = {}
    local step2 = {}
    local step3 = {}
    local fNum = 0

    local mtlTable = {}
    local mtlNum = 0
    local mtlFIlePath = ""

    local startsWithV = {} -- Vertexes
    local startsWithF = {} -- Faces

    local points = {} -- Converted vertexes
    local lines = {} -- Converted faces


    step1 = split(contents, "\n") -- Split the contents of the OBJ file by the newline character

    for i=1,#step1 do -- Loop through all the contents of the OBJ file
        if (starts(step1[i], "mtllib"))
        then
            mtlFilePath = split(step1[i], "%s")[2]
        end

        if (starts(step1[i], "usemtl"))
        then
            mtlNum = mtlNum + 1
            fNum = 0
        end

        if (starts(step1[i], "v ")) -- If it's a vertex, put it in the startsWithV table
        then
            table.insert(startsWithV, step1[i])
        end

        if (starts(step1[i], "f")) -- If it's a face, put it in the startsWithF table
        then
            fNum = fNum + 1
            table.insert(startsWithF, step1[i])
            table.insert(mtlTable, {fNum, mtlNum})
        end

        if (starts(step1[i], "o")) -- If it starts with o, save it in the name variable
        then
            name = split(step1[i], "%s")[2]
        end

    end

    for i=1,#startsWithV do -- Split the variable name and the value

        step2[i] = split(startsWithV[i], "%s")
        table.remove(step2[i], 1)
        local VX = tonumber(step2[i][1]) -- Convert the strings to numbers
        local VY = tonumber(step2[i][2])
        local VZ = tonumber(step2[i][3])

        local point = {}

        point[1] = VX * 40 -- Convert the values to pixel values
        point[2] = VY * 40
        point[3] = VZ * 40

        table.insert(points, point)
    end
    for i=1,#startsWithF do -- Split the variable name and the value
        step3[i] = split(startsWithF[i], "%s")
        table.remove(step3[i], 1)
    end
    for i=1,#startsWithF do
        local p1 = tonumber(split(step3[i][1], "/")[1])
        local p2 = tonumber(split(step3[i][2], "/")[1])
        local p3 = tonumber(split(step3[i][3], "/")[1])
        local _table = {p1, p2, p3, i}
        table.insert(lines, _table)
    end
    local _faceColors = parseMtl(mtlFilePath, mtlTable)
    return {points, lines, name, size, _faceColors}
    
end

function parseMtl(mtlFilePath, mtlTbl) -- Parses the mtl file at mtlFIlePath with 
    local step1 = {}

    local mtlID = 0
    local mtlList = {}
    local faceColors = {}
    contents, size = love.filesystem.read(mtlFilePath, all)
    step1 = split(contents, "\n")
    for i=1,#step1 do
        if (starts(step1[i], "newmtl"))
        then
            mtlID = mtlID + 1
            mtlName = split(step1[i], "%s")[2]
            table.insert(mtlList, {mtlName, mtlID})
        end
        if (starts(step1[i], "Kd"))
        then
            
            R = split(step1[i], "%s")[2] * 255
            G = split(step1[i], "%s")[3] * 255
            B = split(step1[i], "%s")[4] * 255
            mtlList[mtlID] = {mtlList[mtlID][1], mtlList[mtlID][2], {R, G, B}}
        end
        for i=1,#mtlTbl do
            fR = mtlList[mtlTbl[i][2]][1] -- Delete the [1] to not error
            fG = mtlList[mtlTbl[i][2]]
            fB = mtlList[mtlTbl[i][2]]
        end
    end
    return mtlList
end
POBJ.parseObj = parseObj
return POBJ        